/*
===============================================================================
===============================================================================

file:   Buffers.c
by:     Lukasz Krzak & Marcin Krzesinski
date:   16.07.2004

description:   File contains buffer handling functions


===============================================================================
===============================================================================
*/

#define UCOS_PROJECT


#ifdef UCOS_PROJECT
#warning "Buffers: Compiling uC/OS-II version"
#include <includes.h>
#else
#include<avr/stdio.h>
#include<avr/stdlib.h>
#include<Buffers.h>
#endif



/*
-------------------------------------------------------------------------------
CreateBuffer
~~~~~~~~~~~~

This function creates and initializes a buffer of a specified size.
The buffer is arranged as a cyclic FIFO queue.

Parameters:

  int8_t  BufferSize [i] - size of buffer
  
Returns:  pointer to a newly created buffer descriptor structure (TBuffer)
          or NULL if buffer creation failed.

-------------------------------------------------------------------------------
*/
TBuffer *CreateBuffer(int8_t BufferSize    /* size of buffer */
                     )
{
  TBuffer *NewBuffer;
  
  /* allocate memory for buffer descriptor structure: */
  NewBuffer = malloc(sizeof(TBuffer));
  /* check malloc correctness: */
  if (NewBuffer == NULL) return NULL;
  /* allocate memory for buffer: */
  (*NewBuffer).BufferPtr = malloc(BufferSize);
  /* check malloc correctness: */
  if ((*NewBuffer).BufferPtr == NULL)
    {
      free(NewBuffer);  // free memory allocated for descriptor structure
      return NULL;      // return NULL pointer
    }
  /* store buffer size: */
  (*NewBuffer).Size = BufferSize;
  /* set read and write pointers: */
  (*NewBuffer).WritePtr = (*NewBuffer).BufferPtr;
  (*NewBuffer).ReadPtr = (*NewBuffer).BufferPtr;
    /* set initial buffer count */
  (*NewBuffer).Count = 0;

  /* if everything OK return pointer to newly created buffer descriptor: */
  return NewBuffer;
} /* CreateBuffer */


/*
-------------------------------------------------------------------------------
WriteBuffer
~~~~~~~~~~~

This function writes one byte to selected buffer.

Parameters:

  TBuffer  *Buffer    [i] - pointer to buffer descriptor
  int8_t  Byte        [i] - a byte to write

Returns:  error code:              description:
          BUFFER_OK                if everything went OK
          BUFFER_NOT_INITIALIZED   if attempting to write to an uninitialized
                                   buffer
          BUFFER_OVERRUN           if buffer overrun error occured

-------------------------------------------------------------------------------
*/
int8_t WriteBuffer (TBuffer *Buffer,
                    char Byte)
{
    if (Buffer == NULL)                 // check if buffer descriptor initialized
      return BUFFER_NOT_INITIALIZED;
    if ((*Buffer).BufferPtr == NULL)    // check if buffer initialized
      return BUFFER_NOT_INITIALIZED;

    if ((*Buffer).Count == (*Buffer).Size) // check if there's a room in buffer
    return BUFFER_OVERRUN;

    memset((*Buffer).WritePtr,Byte,1);  // write one byte to buffer

    /* update write pointer: */
    if ((*Buffer).WritePtr == (*Buffer).BufferPtr+(*Buffer).Size-1)
      (*Buffer).WritePtr = (*Buffer).BufferPtr;
    else
      (*Buffer).WritePtr++;

    /* increment buffer count */
    (*Buffer).Count++;

  return BUFFER_OK;
} /* WriteBuffer */



/*
-------------------------------------------------------------------------------
ReadBuffer
~~~~~~~~~~

This function reads data of specified width from selected buffer.

Parameters:

  TBuffer  *Buffer     [i] - pointer to buffer descriptor
  char   *Destination  [o] - pointer to memory region where data will be stored
  int8_t  Count        [i] - number of bytes to read

Returns:  error code:              description:
          BUFFER_OK                if everything went OK
          BUFFER_NOT_INITIALIZED   if attempting to write to an uninitialized
                                   buffer
          BUFFER_EMPTY             if buffer was emptied before/while reading
          
-------------------------------------------------------------------------------
*/
int8_t ReadBuffer (TBuffer *Buffer,
                   char *Destination,
                   int8_t Count)
{
  int8_t n,i;

    if (Buffer == NULL)                 // check if buffer descriptor initialized
      return BUFFER_NOT_INITIALIZED;
    if ((*Buffer).BufferPtr == NULL)    // check if buffer initialized
      return BUFFER_NOT_INITIALIZED;

    /* Trim number of bytes to read if it's greater than actual buffer count */
    if (Count > (*Buffer).Count)
         n = (*Buffer).Count;
    else
         n = Count;

    for (i=0;i<n;i++)
    {
      memcpy(Destination,(*Buffer).ReadPtr,1); /* copy byte after byte */
      (*Buffer).Count--;  /* decrement buffer count */

      /* update buffer read pointer */
      if ((*Buffer).ReadPtr == (*Buffer).BufferPtr+(*Buffer).Size-1)
        (*Buffer).ReadPtr = (*Buffer).BufferPtr;
      else
        (*Buffer).ReadPtr++;
    }

  /* if buffer was emptied return proper error code */
  if ((*Buffer).Count == 0) return BUFFER_EMPTY;

  return BUFFER_OK;
} /* ReadBuffer */


/*
-------------------------------------------------------------------------------
BufferCount
~~~~~~~~~~~

This function returns number of unread bytes stored in selected buffer
or 0 if the buffer wasn't initialized

Parameters:

  TBuffer  *Buffer     [i] - pointer to buffer descriptor

Returns:

  int8_t  - number of unread bytes stored in receive buffer
            or 0 if the buffer wasn't initialized

-------------------------------------------------------------------------------
*/
int8_t BufferCount(TBuffer *Buffer)
{
  if (Buffer == NULL)      // check if buffer descriptor initialized..
    return 0;              // ..and return 0 if not

  return (*Buffer).Count;  // else return current buffer count
  
} /* BufferCount */
