/*
===============================================================================
===============================================================================

file:   LCD.c
by:     Lukasz Krzak & Marcin Krzesinski
date:   28.06.2004

description:   This file contains procedures and functions used to handle
               a LCD unit.
               At this point it is designed to work with 16x2 LCD in 4bit modus

===============================================================================
===============================================================================
*/

/* The following definition causes compilation of a uC/OS-II compatible version.
   For standard applications this should be removed. */
#define UCOS_PROJECT

/* For uC/OS version only file "includes.h" is included. For non-uC/OS projects
   a standard set of header files is included. */
#ifdef UCOS_PROJECT
#warning "LCD: Compiling uC/OS-II version"
#include <includes.h>
#else
#include<avr/io.h>
#include<avr/stdio.h>
#include<avr/stdlib.h>
#include<avr/delay.h>
#include<string.h>
#include<LCD.h>
#endif





/*
-------------------------------------------------------------------------------
delay
~~~~~

Parameters:

  int msecdel [i]   - delay (at this moment NOT in milisec units)

-------------------------------------------------------------------------------
*/
void delay(int msecdel)
{
   int i;
   for (i=0;i<msecdel;i++)
   {
     _delay_loop_2(LOOP_TIME*500);
   }
} /* delay */


/*
-------------------------------------------------------------------------------
LCD_WriteByte
~~~~~~~~~~~~~

This procedure sends one byte to the LCD unit

Parameters:

  int8_t LCDByte [i] - Byte to send
  int8_t IsData  [i] - This should be 0 if byte contains control instruction
                                   or 1 if byte contains data

-------------------------------------------------------------------------------
*/
void LCD_WriteByte(int8_t LCDByte, int8_t IsData)
{
   #ifdef UCOS_PROJECT     // if it's a uC/OS project then make sure that
   OS_ENTER_CRITICAL();    // the following code is indivisible
   #endif
   /* setting apropriate RS signal depending on whether we send instructions
      or data */
   if (IsData) sbi(LCD_PORT,LCD_RS);
   else cbi(LCD_PORT,LCD_RS);
   /* writing more significant 4bit piece on LCD ports */
   if  (LCDByte & 0x80) sbi(LCD_PORT,LCD_DB7);
   else cbi(LCD_PORT,LCD_DB7);
   if  (LCDByte & 0x40) sbi(LCD_PORT,LCD_DB6);
   else cbi(LCD_PORT,LCD_DB6);
   if  (LCDByte & 0x20) sbi(LCD_PORT,LCD_DB5);
   else cbi(LCD_PORT,LCD_DB5);
   if  (LCDByte & 0x10) sbi(LCD_PORT,LCD_DB4);
   else cbi(LCD_PORT,LCD_DB4);
   /* data enable strobe */
   sbi(LCD_PORT,LCD_E);
   delay(5);
   cbi(LCD_PORT,LCD_E);
   delay(5);
   /* writing less significant 4bit piece on LCD ports */
   if  (LCDByte & 0x08) sbi(LCD_PORT,LCD_DB7);
   else cbi(LCD_PORT,LCD_DB7);
   if  (LCDByte & 0x04) sbi(LCD_PORT,LCD_DB6);
   else cbi(LCD_PORT,LCD_DB6);
   if  (LCDByte & 0x02) sbi(LCD_PORT,LCD_DB5);
   else cbi(LCD_PORT,LCD_DB5);
   if  (LCDByte & 0x01) sbi(LCD_PORT,LCD_DB4);
   else cbi(LCD_PORT,LCD_DB4);
   /* data enable strobe */
   sbi(LCD_PORT,LCD_E);
   delay(5);
   cbi(LCD_PORT,LCD_E);
   delay(5);
   #ifdef UCOS_PROJECT     // if it's a uC/OS project then
   OS_EXIT_CRITICAL();     // close the critical section
   #endif

} /* LCD_WriteByte */


/*
-------------------------------------------------------------------------------
LCD_WriteText
~~~~~~~~~~~~~

This procedure sends a string of text to the LCD unit

Parameters:

  char text [i] - a null terminated string containing text to be displayed

-------------------------------------------------------------------------------
*/
void LCD_WriteText(char text[])
{
   int8_t i=0;
   while (text[i] != 0)
   {
     LCD_WriteByte(text[i],1);
     i++;

   }
} /* LCD_WriteText */


/*
-------------------------------------------------------------------------------
LCD_Goto
~~~~~~~~

This procedure sets the cursor in desired position

Parametry:

  int8_t column  [i] - column (first one = 0)
  int8_t line    [i] - line (first one = 0)

-------------------------------------------------------------------------------
*/
void LCD_Goto(int8_t column, int8_t line)
{
  LCD_WriteByte(((line * 64) + column) | 0x80,0);
}  /* LCD_Goto */


/*
-------------------------------------------------------------------------------
LCD_ClrLine
~~~~~~~~~~~

this procedure clears one line on the LCD unit

Parameters:

  int8_t line  [i] - line to be cleared (first one = 0)

-------------------------------------------------------------------------------
*/
void LCD_ClrLine(int8_t line)
{
   int8_t i;
   LCD_Goto(0,line);    // set cursor at the beginning of line
   for (i=0;i<16;i++)   // for entire line
    {
      LCD_WriteByte(0x20,1);   // send ' ' (space) character
    }
} /* LCD_ClrLine */

/*
-------------------------------------------------------------------------------
LCD_Init
~~~~~~~~

This procedure initializes the LCD unit

Parameters: none

At this moment this procedure sets the LCD unit in 4bit modus only

-------------------------------------------------------------------------------
*/
void LCD_Init(void)
{

   /* setting the proper direction of AVR I/O ports (see #define section
      in LCD.h header file) */
   LCD_DDR |= (0x01 << LCD_DB7);
   LCD_DDR |= (0x01 << LCD_DB6);
   LCD_DDR |= (0x01 << LCD_DB5);
   LCD_DDR |= (0x01 << LCD_DB4);
   LCD_DDR |= (0x01 << LCD_RS);
   LCD_DDR |= (0x01 << LCD_E);

   /* instruction initializing the LCD display unit: */
   LCD_WriteByte(0x33,0);
   LCD_WriteByte(0x32,0);
   LCD_WriteByte(0x28,0);   // set interface length (4bit)
   LCD_WriteByte(0x0c,0);   // enable display
   LCD_WriteByte(0x01,0);   // clear display
   delay(10);
   LCD_WriteByte(0x06,0);   // set cursor move direction
   delay(10);
} /* LCDInit */



/*
-------------------------------------------------------------------------------
LCD_BarGraphInit
~~~~~~~~~~~~~~~~

This procedure initializes a horizontal bar graph. It defines a set of characters
in LCD's CGRAM at locations: 0x00..0x03

Parameters: none

-------------------------------------------------------------------------------
*/

void LCD_BarGraphInit(void)
{
  int8_t i;
  LCD_WriteByte(0x40,0);
  for (i=0;i<8;i++)         // 0x00
  {
    LCD_WriteByte(0x10,1);
  }
   for (i=0;i<8;i++)        // 0x01
  {
    LCD_WriteByte(0x18,1);
  }
   for (i=0;i<8;i++)        // 0x02
  {
    LCD_WriteByte(0x1c,1);
  }
   for (i=0;i<8;i++)        // 0x03
  {
    LCD_WriteByte(0x1e,1);
  }
} /* LCD_BarGraphInit */


/*
-------------------------------------------------------------------------------
LCD_BarGraph
~~~~~~~~~~~~

This procedure displays a horizontal bar graph on the LCD unit in desired line.
The graph is 10 characters wide. Value is displayed at the end of line witch
'%' character.
Before using this function application must call LCD_BarGraphInit.


Parameters:

  int8_t line  [i] - desired bar graph line (first one = 0)
  int8_t value [i] - value to be displayed (0-100)

-------------------------------------------------------------------------------
*/
void LCD_BarGraph(int8_t line, int8_t value)
{
  int8_t WholeBars,Rest,tmp,i;
  char text[5] = {' ',' ',' ',' ',0};

  /* error message if value out of bounds */
  if (value > 100)
  {
     LCD_Goto(0,line);
     LCD_WriteText("Bar Value Error");
  }
  /* display all "full" characters */
  LCD_Goto(0,line);
  WholeBars = (int8_t)(value / 10);  /* 1 pixel = 2% */
  if (WholeBars > 0)
  for (i=0;i<WholeBars;i++)
  {
    LCD_WriteByte(0xff,1);
  }
  
  /* display optional character partially filled  */
  Rest = (int8_t)((value % 10) /2);
  if (Rest > 0)
     LCD_WriteByte(Rest,1);   /* number of filled pixels is also a character id */

  /* calculate the number of unused charaters in line */
  tmp = 10 - WholeBars;
  if (Rest > 0) tmp--;

  /* clear  the unused characters in line */
  if (tmp > 0)
    for (i=0;i<tmp;i++)
      LCD_WriteByte(' ',1);

  /* clear the value display region */
  LCD_Goto(12,line);
  LCD_WriteText(text);
  /* display the value with '%' character */
  itoa(value,text,10);
  LCD_Goto(15-strlen(text),line);
  LCD_WriteText(text);
  LCD_Goto(15,line);
  LCD_WriteByte('%',1);
  
} /* LCD_BarGraph */
