/*
===============================================================================
===============================================================================

file:   USART.c
by:     Lukasz Krzak & Marcin Krzesinski
date:   29.06.2004

description:   File contains USART service functions


===============================================================================
===============================================================================
*/

/* The following definition causes compilation of a uC/OS-II compatible version.
   For standard applications this should be removed. */

#define UCOS_PROJECT

/* For uC/OS version only file "includes.h" is included. For non-uC/OS projects
   a standard set of header files is included. */
#ifdef UCOS_PROJECT
#warning "USART: Compiling uC/OS-II version"
#include <includes.h>
#else
#include<avr/stdio.h>
#include<avr/stdlib.h>
#include<avr/io.h>
#include<avr/pgmspace.h>
#include<Buffers/Buffers.h>
#include<USART.h>
#endif


#define USART0_RXD_ISR()   SIGNAL(SIG_UART0_RECV)
#define USART1_RXD_ISR()   SIGNAL(SIG_UART1_RECV)

/* TYPES: */


/* VARIABLES: */
/* The following 2 variables hold current options for USARTs
   The meaning of bits is:
   
   USARTx_OPTIONS:  [ b7 , b6 , b5 , b4 , b3 , b2 , b1 , b0 ] (8bit)
   
       b7 - set only if USARTx is initialized
       
       b3 - set if RXD buffering is enabled
       b2 - set if TXD buffering is enabled
*/
int8_t USART0_Options;
int8_t USART1_Options;

TBuffer *RXD0Buffer; /* USART0 Receive-Buffer descriptor*/
TBuffer *RXD1Buffer; /* USART1 Receive-Buffer descriptor*/

static prog_char BufferOverrun[]="Buffer overrun error!\n\r";

//TUSARTBuffer *TXD0Buffer; /* USART0 Send-Buffer descriptor*/
//TUSARTBuffer *TXD1Buffer; /* USART1 Send-Buffer descriptor*/


/*
-------------------------------------------------------------------------------
USART_Init
~~~~~~~~~~

This function initializes the specified USART according to the following
parameters

Parameters:

  int8_t  USARTNo       [i] - USARTu number (currently only 0 or 1)
  int8_t  USARTMode     [i] - asynchronous / synchronous mode
  int16_t BaudRatePresc [i] - clock prescaler used to set the transmission speed
  int8_t  CharSize      [i] - bits per char (5-9)
  int8_t  PSOptions     [i] - parity and stop bit options
  int8_t  IEOptions     [i] - interrupt enable options
  int8_t  OtherOptions  [i] - additional options

-------------------------------------------------------------------------------
*/
void USART_Init(int8_t  USARTNo,           /* USARTu number                  */
                int8_t  USARTMode,         /* Async/Sync                     */
                int16_t BaudRatePresc,     /* clock prescaler                */
                int8_t  CharSize,          /* bits per char (5-9)            */
                int8_t  PSOptions,         /* parity and stop bit options    */
                int8_t  IEOptions,         /* interrupt enable options       */
                int8_t  OtherOptions       /* additional options             */
                )
{
  if (USARTNo == USART0) // for USART0:
  {
    /* set the UCSR0C register with selected options:      */
    switch (CharSize)
    {
      case 5:  UCSR0C = (int8_t)(PSOptions); break;
      case 6:  UCSR0C = (int8_t)(PSOptions | 0x02); break;
      case 7:  UCSR0C = (int8_t)(PSOptions | 0x04); break;
      default:
      case 8:
      case 9:  UCSR0C = (int8_t)(PSOptions | 0x06); break;
    }

    /*  set the UCSR0B register with interrupt enable options:   */
    if (CharSize == 9)
      UCSR0B = (int8_t)(IEOptions | 0x1C);
    else
      UCSR0B = (int8_t)(IEOptions | 0x18);

    /* set the UCSR0A register with additional options:      */
    UCSR0A = (int8_t)(OtherOptions & 0xf3);
    /* set the transmision baud rate clock prescaler: */
    UBRR0H = (int8_t)(BaudRatePresc>>8);
    UBRR0L = (int8_t)(BaudRatePresc);
    
    if ((OtherOptions & USART_RXDBuffer) == USART_RXDBuffer) // if RXD buffering enabled
    {
      RXD0Buffer = CreateBuffer(USART0_RXD_BUFFER_SIZE); // create RXD buffer
    }
    
    /* set global configuration variable: */
    USART0_Options = 0x80 | OtherOptions;

  } // USARTNo = USART0
  
  if (USARTNo == USART1)
  {
    /* set the UCSR1C register with selected options:      */
    
    switch (CharSize)
    {
      case 5:  UCSR1C = (int8_t)(PSOptions); break;
      case 6:  UCSR1C = (int8_t)(PSOptions | 0x02); break;
      case 7:  UCSR1C = (int8_t)(PSOptions | 0x04); break;
      default:
      case 8:
      case 9:  UCSR1C = (int8_t)(PSOptions | 0x06); break;
    }

    /* set the UCSR1B register with interrupt enable options:      */
    if (CharSize == 9)
      UCSR1B = (int8_t)(IEOptions | 0x1C);
    else
      UCSR1B = (int8_t)(IEOptions | 0x18);

    /* Uset the UCSR1A register with additional options:      */
    UCSR1A = (int8_t)(OtherOptions & 0xf3);
    /* set the transmision baud rate clock prescaler: */
    UBRR1H = (int8_t)(BaudRatePresc>>8);
    UBRR1L = (int8_t)(BaudRatePresc);

    if ((OtherOptions & USART_RXDBuffer) == USART_RXDBuffer) // if RXD buffering enabled
    {
      RXD1Buffer = CreateBuffer(USART1_RXD_BUFFER_SIZE); // create RXD buffer
    }

    /* set global configuration variable: */
    USART0_Options = 0x80 | OtherOptions;

  } // USARTNo = USART1



} /* USART_Init */


/*
-------------------------------------------------------------------------------
USART_GetChar
~~~~~~~~~~~~~

This function reads one character from selected USART's read-buffer

Parameters:

  int8_t  USARTNo       [i] - USART number (currently 0 or 1 only)

Returns: 8bit character (optional 9th bit is stored in apropriate AVR register)

-------------------------------------------------------------------------------
*/
int8_t USART_GetChar(int8_t  USARTNo          /* USART number  */
                    )
{
  /* if USART0 selected: */
  if (USARTNo == USART0)
  {
     while (!(UCSR0A & 0x80))  /* wait for Receive Complete flag */
       ;
     return (int8_t)UDR0;      /* read buffer */
  }
  /* if USART1 selected: */
  if (USARTNo == USART1)
  {
     while (!(UCSR1A & 0x80)) /* wait for Receive Complete flag */
       ;
     return (int8_t)UDR1;     /* read buffer */
  }
  return 0;
} /* USART_GetChar */


/*
-------------------------------------------------------------------------------
USART_PutChar
~~~~~~~~~~~~~

This function sends one character to selected USART's send-buffer.

Parameters:

  int8_t  USARTNo       [i] - USART number (currently 0 or 1 only)
  char    Character     [i] - character to send

-------------------------------------------------------------------------------
*/
void USART_PutChar(int8_t USARTNo,           /* USART number      */
                     char Character          /* character to send */
                     )
{
   /* if USART0 selected: */
  if (USARTNo == USART0)
  {
    while ((UCSR0A & 0x20) == 0) /* wait for Data Register Empty flag*/
      ;
    UDR0 = Character;   /* write to buffer */
  }
  /* if USART1 selected: */
  if (USARTNo == USART1)
  {
    while ((UCSR1A & 0x20) == 0) /* wait for Data Register Empty flag*/
      ;
    UDR1 = Character;   /* write to buffer */
  }
} /* USART_PutChar */


/*
-------------------------------------------------------------------------------
USART_PutStr
~~~~~~~~~~~~~

This procedure sends a null-terminated string out through selected USART

Parameters:

  int8_t  USARTNo    [i] - USART number (currently 0 or 1 only)
  char[]  String     [i] - a null-terminated string of characters to send
  int8_t  Location   [i] - type of memory the string is located in, this can be:
                           MEM_RAM , MEM_EEPROM , MEM_FLASH

-------------------------------------------------------------------------------
*/
void USART_PutStr(int8_t USARTNo,           /* USART number            */
                     const char String[],   /* characters to send      */
                     int8_t Location        /* type of memory          */
                     )
{
  int8_t i=0;
  char FlashByte;
  
  if (Location == MEM_RAM)   // characters in RAM memory
  {
    while(String[i] != 0)
    {
      USART_PutChar(USARTNo,String[i]);
      i++;
    }
  }
  if (Location == MEM_EEPROM)
  {
    // TODO: EEPROM
  }
  if (Location == MEM_FLASH)  // characters in program memory (flash)
  {
    for (;;)
    {
      FlashByte = pgm_read_byte_near(String+i);
      if (FlashByte != 0)
        {
           USART_PutChar(USARTNo,FlashByte);
           i++;
        }
      else return;
    }
  }
} /* USART_PutStr */


/*
-------------------------------------------------------------------------------
USART_DataReceived
~~~~~~~~~~~~~~~~~~

This function returns 1 if there is an unread character in hardware buffer,
otherwise it returns 0.
ATTENTION: This function should not be used if buffering is enabled - use
           USART_RXDBufferCount instead.

Parameters:

  int8_t  USARTNo    [i] - USART number (currently 0 or 1 only)

Returns:
        0 if no new characters were received
        1 if there's a new character waiting in receive-buffer

-------------------------------------------------------------------------------
*/
int8_t USART_DataReceived(int8_t USARTNo    /* USARTu number    */
                          )
{
  if (USARTNo == USART0)
  {
   if ((UCSR0A & 0x80) != 0) return 1;
   else return 0;
  }

  if (USARTNo == USART1)
  {
    if ((UCSR1A & 0x80) != 0) return 1;
    else return 0;
  }
  return 0;
} /* USART_DataReceived */

/*
-------------------------------------------------------------------------------
SystemLog
~~~~~~~~~

This procedure sends a system log through LOG_USART defined in USART.h
ATTENTION: The text of log must be stored in flash(program) memory.

We use FLASH memory to save on RAM operational memory which would be quickly
used up only for system logs.

Parameters:

  char  *String [i] - address of log text in FLASH memory

-------------------------------------------------------------------------------
*/
void SystemLog(const char *String  // system log text
              )
{
  #ifdef UCOS_PROJECT  // only for uC/OS applications:
  if (VAR_SystemLogs != 0)
  #endif
    USART_PutStr(LOG_USART,String,MEM_FLASH);
} /* SystemLog */



/*
-------------------------------------------------------------------------------
USART0_RXD_ISR
~~~~~~~~~~~~~~

This is an interrupt service routine for USART0 Character Received interrupt.

Parameters: none

-------------------------------------------------------------------------------
*/
USART0_RXD_ISR()
{
  int8_t Error;
  if ((USART0_Options &= USART_RXDBuffer) == USART_RXDBuffer) // if RXD0 buffering enabled..
  {
     /* read from hardware buffer (this also clears RXD interrupt flag) */
     Error = USART_WriteRXDBuffer(USART0,USART_GetChar(USART0));
     if (Error == BUFFER_OVERRUN )
     {
        OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource
        SystemLog(BufferOverrun);
        OSMutexPost(USART0Mutex); //return access to USART0 resource
     }
  }
  else
    /* read from hardware buffer (to clear RXD interrupt flag) but flush the data*/
    USART_GetChar(USART0);
  
} /* USART0_RXD_ISR */

/*
-------------------------------------------------------------------------------
USART1_RXD_ISR
~~~~~~~~~~~~~~

This is an interrupt service routine for USART1 Character Received interrupt.

Parameters: none

-------------------------------------------------------------------------------
*/
USART1_RXD_ISR()
{
  int8_t Error;
  if ((USART1_Options &= USART_RXDBuffer) == USART_RXDBuffer) // if RXD1 buffering enabled..
  {
     /* read from hardware buffer (this also clears RXD interrupt flag) */
     Error = USART_WriteRXDBuffer(USART1,USART_GetChar(USART1));
     if (Error == BUFFER_OVERRUN )
     {
        OSMutexPend(USART1Mutex,0,NULL); //  gain access to USART1 resource
        SystemLog(BufferOverrun);
        OSMutexPost(USART1Mutex); //return access to USART1 resource
     }
  }
  else
    /* read from hardware buffer (to clear RXD interrupt flag) but flush the data*/
    USART_GetChar(USART1);

} /* USART1_RXD_ISR */

/*
-------------------------------------------------------------------------------
USART_WriteRXDBuffer
~~~~~~~~~~~~~~~~~~~~~

This function writes one byte to selected USART's receive buffer.
ATTENTION: This function should never be called from outside of this module.
           It is used in Data Received ISR to copy data from hardware buffer
           to USART's receive buffer.

Parameters:

  int8_t  USARTNo    [i] - USART number (currently 0 or 1 only)
  int8_t  Byte       [i] - a byte to write

Returns:  error code:                     description:
          USART_OK                        if everything went OK
          USART_BUFFER_NOT_INITIALIZED    if attempting to write to an uninitialized
                                          buffer
          USART_BUFFERING_DISABLED        if buffering wasn't enabled
          USART_OVERRUN                   if buffer overrun error occured
          
-------------------------------------------------------------------------------
*/
int8_t USART_WriteRXDBuffer (int8_t USARTNo,
                            char Byte)
{
  if (USARTNo == USART0 )
  {
    if ((USART0_Options &= USART_RXDBuffer) != USART_RXDBuffer) // if RXD0 buffering disabled..
    return USART_BUFFERING_DISABLED;                          // ..return proper error code
    return WriteBuffer(RXD0Buffer,Byte); // write RXD0 buffer and return
  }
  if (USARTNo == USART1 )
  {
    if ((USART1_Options &= USART_RXDBuffer) != USART_RXDBuffer) // if RXD1 buffering disabled..
    return USART_BUFFERING_DISABLED;                          // ..return proper error code
    return WriteBuffer(RXD1Buffer,Byte); // write RXD1 buffer and return
  }
  return USART_OK;
} /* USART_WriteRXDBuffer */



/*
-------------------------------------------------------------------------------
USART_ReadRXDBuffer
~~~~~~~~~~~~~~~~~~~

This function reads data of specified width from selected USART's receive buffer.

Parameters:

  int8_t  USARTNo      [i] - USART number (currently 0 or 1 only)
  char   *Destination  [o] - pointer to memory region where data will be stored
  int8_t  Count        [i] - number of bytes to read

Returns:  error code:                     description:
          USART_OK                        if everything went OK
          USART_BUFFER_NOT_INITIALIZED    if attempting to write to an uninitialized
                                          buffer
          USART_BUFFERING_DISABLED        if buffering wasn't enabled
          USART_BUFFER_EMPTY              if buffer was emptied before/while reading
          
-------------------------------------------------------------------------------
*/
int8_t USART_ReadRXDBuffer (int8_t USARTNo,
                            char *Destination,
                            int8_t Count)
{
  if (USARTNo == USART0 )
  {
    if ((USART0_Options &= USART_RXDBuffer) != USART_RXDBuffer) // if RXD0 buffering disabled..
    return USART_BUFFERING_DISABLED;                          // ..return proper error code
    return ReadBuffer(RXD0Buffer,Destination,Count); // else read RXD0 buffer and return
  }
  if (USARTNo == USART1 )
  {
    if ((USART1_Options &= USART_RXDBuffer) != USART_RXDBuffer) // if RXD1 buffering disabled..
    return USART_BUFFERING_DISABLED;                          // ..return proper error code
    return ReadBuffer(RXD1Buffer,Destination,Count); // else read RXD1 buffer and return
  }
  return USART_OK;
} /* USART_ReadRXDBuffer */


/*
-------------------------------------------------------------------------------
USART_RXDBufferCount
~~~~~~~~~~~~~~~~~~~~

This function returns number of unread bytes stored in selected USART's receive
buffer or 0 if buffer wasn't initialized.

Parameters:

  int8_t  USARTNo      [i] - USART number (currently 0 or 1 only)

Returns:

  int8_t  - number of unread bytes stored in receive buffer
            or 0 if buffer wasn't initialized
  
-------------------------------------------------------------------------------
*/
int8_t USART_RXDBufferCount(int8_t USARTNo)
{
  if (USARTNo == USART0 )
  {
     return BufferCount(RXD0Buffer);
  }
  if (USARTNo == USART1 )
  {
     return BufferCount(RXD1Buffer);
  }
  return 0;
} /* USART_RXDBufferCount */
